#include "lbm.h"
const int ex[9] = {0,  1,0,-1, 0,  1,-1,-1, 1};
const int ey[9] = {0,  0,1, 0,-1,  1, 1,-1,-1};
const int inv[9] = {0, 3,4, 1, 2,  7, 8, 5, 6};
const float w[9]={4.0/9.0,  1.0/9.0,1.0/9.0,1.0/9.0,1.0/9.0,   1.0/36.0,1.0/36.0,1.0/36.0,1.0/36.0};
float df[2][L*L][9];
float U[L*L], V[L*L], R[L*L];
int FLAG[L*L];
float fx = 0.000014;
float omega = 1.98;

void initlbm(void)
{
	for(int j=0; j < L*L ; j++)	
	for(int k=0; k< 9; k++)	
		df [0][ j ][k] = df [1][ j ][k] = w[k];
	const int R=5;
	for(int i=(L-R)/2; i < (L-R)/2 + R; i++)
	for(int j=(L-R)/2; j < (L-R)/2 + R ; j++)
		FLAG[i+j*L] = 1;
	const int R2=8;
	for(int i=(L-R2)/4; i < (L-R2)/4 + R; i++)
	for(int j=(L-R2)/3; j < (L-R2)/3 + R ; j++)
		FLAG[i+j*L] = 1;
}

// we use BGK single relaxation time model with no relaxation on the bounceback nodes Zou, He, 1997
void lbm(void)
{
	static int c = 1;
	c = 1-c;
	for(int i=0 ; i < L ; i++)
	for(int j=0 ; j < L ; j++)
	if(FLAG[ i+j*L ] == 0)
	{
		float rho=0,ux=0,uy=0;
		for(int k=0; k<9; k++)			// calculate density and velocity
		{
			rho = rho + df[c][ i+j*L ][k];
			ux = ux + df[c][ i+j*L ][k] * ex[k];
			uy = uy + df[c][ i+j*L ][k] * ey[k];
		}
		ux /= rho;
		uy /= rho;
		U[ i+j*L ] = ux;
		V[ i+j*L ] = uy;
		R[ i+j*L ] = rho;
		ux = ux + fx;
		for(int k=0; k< 9; k++)
		{
			int ip = ( i+ex[k] + L ) % (L);
			int jp = ( j+ey[k] + L ) % (L);
			// collision + streaming
			if( FLAG[ip+jp*L] == 1 )
				df [1-c][ i+j*L ][inv[k]] = (1-omega) * df[c][ i+j*L ][k] + omega* w[k] * rho * (1.0f - (3.0f/2.0f) * (ux*ux + uy*uy) + 3.0f * (ex[k] * ux + ey[k]*uy) + (9.0f/2.0f) * (ex[k] * ux + ey[k]*uy) * (ex[k] * ux + ey[k]*uy));		
			else
				df [1-c][ip+jp*L][k] = (1-omega) * df[c][ i+j*L ][k] + omega* w[k] * rho * (1.0f - (3.0f/2.0f) * (ux*ux + uy*uy)+ 3.0f * (ex[k] * ux + ey[k]*uy) + (9.0f/2.0f) * (ex[k] * ux + ey[k]*uy) * (ex[k] * ux + ey[k]*uy));					
		}
	} 
}
